/*!
 *  @file Adafruit_MPL3115A2.cpp
 *
 *  @mainpage Adafruit MPL3115A2 alitmeter
 *
 *  @section intro_sec Introduction
 *
 *  This is the documentation for Adafruit's MPL3115A2 driver for the
 *  Arduino platform.  It is designed specifically to work with the
 *  Adafruit MPL3115A2 breakout: https://www.adafruit.com/products/1893
 *
 *  These sensors use I2C to communicate, 2 pins (SCL+SDA) are required
 *  to interface with the breakout.
 *
 *  Adafruit invests time and resources providing this open source code,
 *  please support Adafruit and open-source hardware by purchasing
 *  products from Adafruit!
 *
 *  @section dependencies Dependencies
 *
 *  @section author Author
 *
 *  Written by Kevin "KTOWN" Townsend for Adafruit Industries.
 *
 *  @section license License
 *
 *  BSD license, all text here must be included in any redistribution.
 *
 */

#include <xc.h>
#include "i2c4mz.h"
#include "Adafruit_MPL3115A2.h"


void MPL3115A2_init(){
    uint8_t whoami = read8(MPL3115A2_WHOAMI);
    if (whoami != 0xC4) {
        return;
    }
    
    write8(MPL3115A2_CTRL_REG1, MPL3115A2_CTRL_REG1_RST);
    while (read8(MPL3115A2_CTRL_REG1) & MPL3115A2_CTRL_REG1_RST){
      uint16_t i;
      for(i=0;i<65000;i++);
    }
    
    ctrl_reg1 _ctrl_reg1;
    _ctrl_reg1.reg = MPL3115A2_CTRL_REG1_OS1 | MPL3115A2_CTRL_REG1_ALT;
    write8(MPL3115A2_CTRL_REG1, _ctrl_reg1.reg);
    write8(MPL3115A2_CTRL_REG4, 0x00); // disable all interrupts
    write8(MPL3115A2_CTRL_REG3, MPL3115A2_CTRL_REG3_IPOL1); // ensures INT1 interrupts operate in push-pull mode
    write8(MPL3115A2_CTRL_REG5, MPL3115A2_CTRL_REG5_DRDY); // data ready interrupt is mapped to INT1 pin
    write8(MPL3115A2_PT_DATA_CFG, MPL3115A2_PT_DATA_CFG_PDEFE); // enables event flag for pressure/altitude data ready   
    
    /* PIC32 Interrupt Setup Here */
    
    write8(MPL3115A2_CTRL_REG4, MPL3115A2_CTRL_REG4_DRDY); // enables interrupt caused by data ready event 
}

uint16_t readAltitude(){ // This code will be placed in the External ISR
    uint8_t sta;
    uint16_t alt;
    do {
        sta = read8(MPL3115A2_REGISTER_STATUS);
    } while(!(sta & MPL3115A2_REGISTER_STATUS_PDR));
    I2C4_start();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x00); // start transmission to device
    I2C4_write(MPL3115A2_REGISTER_PRESSURE_MSB);               // sends register address to read from
    I2C4_restart();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x01);
    alt = I2C4_read(ACK); 
    alt <<= 8;
    alt |= I2C4_read(ACK); 
    alt <<= 8;
    alt |= I2C4_read(NACK); 
    I2C4_stop(); 
    return alt;
}

/*!
 *   @brief  Setups the HW (reads coefficients values, etc.)
 *   @param  twoWire 
 *           Optional TwoWire I2C object
 *   @return true on successful startup, false otherwise
 */
uint8_t begin(void) {
  uint8_t whoami = read8(MPL3115A2_WHOAMI);
  if (whoami != 0xC4) {
    return FALSE;
  }

  write8(MPL3115A2_CTRL_REG1, MPL3115A2_CTRL_REG1_RST);
  //delay_ms(5);

  while (read8(MPL3115A2_CTRL_REG1) & MPL3115A2_CTRL_REG1_RST){
      int i;
      for(i=0;i<10000;i++);
  }

  ctrl_reg1 _ctrl_reg1;
  _ctrl_reg1.reg = MPL3115A2_CTRL_REG1_OS1 | MPL3115A2_CTRL_REG1_ALT;

  write8(MPL3115A2_CTRL_REG1, _ctrl_reg1.reg);
  write8(MPL3115A2_CTRL_REG4, 0x00); // disable all interrupts

//  write8(MPL3115A2_PT_DATA_CFG, MPL3115A2_PT_DATA_CFG_TDEFE |
//                                    MPL3115A2_PT_DATA_CFG_PDEFE |
//                                    MPL3115A2_PT_DATA_CFG_DREM);
  
  write8(MPL3115A2_CTRL_REG3, MPL3115A2_CTRL_REG3_IPOL1); // ensures INT1 interrupts operate in push-pull mode
  write8(MPL3115A2_CTRL_REG5, MPL3115A2_CTRL_REG5_DRDY); // data ready interrupt is mapped to INT1 pin
  write8(MPL3115A2_PT_DATA_CFG, MPL3115A2_PT_DATA_CFG_PDEFE); // enables event flag for pressure/altitude data ready
  //delay_ms(5);
  write8(MPL3115A2_CTRL_REG4, MPL3115A2_CTRL_REG4_DRDY); // enables interrupt caused by data ready event
  
  
  return TRUE;
}

/*!
 *  @brief  Gets the floating-point pressure level in kPa
 *  @return altitude reading as a floating point value
 */
float getPressureOS() {
    uint32_t pressure;
    ctrl_reg1 _ctrl_reg1;
    
    do {
        _ctrl_reg1.reg = read8(MPL3115A2_CTRL_REG1);
        //delay_ms(10);
    } while (_ctrl_reg1.reg & MPL3115A2_CTRL_REG1_OST);

    _ctrl_reg1.bits.ALT = 0;
    write8(MPL3115A2_CTRL_REG1, _ctrl_reg1.reg);

    _ctrl_reg1.bits.OST = 1;
    write8(MPL3115A2_CTRL_REG1, _ctrl_reg1.reg);

    uint8_t sta = 0;
    while (!(sta & MPL3115A2_REGISTER_STATUS_PDR)) {
    sta = read8(MPL3115A2_REGISTER_STATUS);
    //delay_ms(10);
    }
    
    I2C4_start();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x00); // start transmission to device
    I2C4_write(MPL3115A2_REGISTER_PRESSURE_MSB);               // sends register address to read from
    I2C4_restart();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x01);
    pressure = I2C4_read(ACK); 
    pressure <<= 8;
    pressure |= I2C4_read(ACK); 
    pressure <<= 8;
    pressure |= I2C4_read(NACK); 
    I2C4_stop();    
    pressure >>= 4;
    
    float baro = pressure;
    baro /= 4.0;
    return baro;
}

/*!
 *  @brief  Gets the floating-point altitude value
 *  @return altitude reading as a floating-point value
 */
float getAltitudeOS() {
    int32_t alt;
    ctrl_reg1 _ctrl_reg1;

    do {
        _ctrl_reg1.reg = read8(MPL3115A2_CTRL_REG1);
        //delay_ms(10);
    } while (_ctrl_reg1.reg & MPL3115A2_CTRL_REG1_OST);

    _ctrl_reg1.bits.ALT = 1;
    write8(MPL3115A2_CTRL_REG1, _ctrl_reg1.reg);

    _ctrl_reg1.bits.OST = 1;
    write8(MPL3115A2_CTRL_REG1, _ctrl_reg1.reg);

    uint8_t sta = 0;        // wait until new altitude data is ready
    while (!(sta & MPL3115A2_REGISTER_STATUS_PDR)) {
        sta = read8(MPL3115A2_REGISTER_STATUS);
        //delay_ms(10);
    }

    I2C4_start();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x00); // start transmission to device
    I2C4_write(MPL3115A2_REGISTER_PRESSURE_MSB);               // sends register address to read from
    I2C4_restart();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x01);
    alt = ((uint32_t)I2C4_read(ACK)) << 24; 
    alt |= ((uint32_t)I2C4_read(ACK)) << 16; 
    alt |= ((uint32_t)I2C4_read(NACK)) << 8; 
    I2C4_stop();       

    float altitude = alt;
    altitude /= 65536.0;
    return altitude;
}

/*!
 *  @brief  Set the local sea level barometric pressure
 *  @param pascal the pressure to use as the baseline
 */
void setSeaPressure(float pascal) {
    uint16_t bar = pascal / 2;
    I2C4_start();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x00); // start transmission to device
    I2C4_write(MPL3115A2_BAR_IN_MSB);
    I2C4_write(bar >> 8);
    I2C4_write(bar);
    I2C4_stop();
}

/*!
 *  @brief  Gets the floating-point temperature in Centigrade
 *  @return temperature reading in Centigrade as a floating-point value
 */
float getTemperatureOS() {
    int16_t t;
    ctrl_reg1 _ctrl_reg1;
    
    do {
        _ctrl_reg1.reg = read8(MPL3115A2_CTRL_REG1);
        //delay_ms(10);
    } while (_ctrl_reg1.reg & MPL3115A2_CTRL_REG1_OST);
    
    _ctrl_reg1.bits.OST = 1;
    write8(MPL3115A2_CTRL_REG1, _ctrl_reg1.reg);

    uint8_t sta = 0;
    while (!(sta & MPL3115A2_REGISTER_STATUS_TDR)) {
        sta = read8(MPL3115A2_REGISTER_STATUS);
        //delay_ms(10);
    }
    
    I2C4_start();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x00); // start transmission to device
    I2C4_write(MPL3115A2_REGISTER_TEMP_MSB);               // sends register address to read from
    I2C4_restart();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x01);
    t = I2C4_read(ACK);
    t <<= 8;
    t |= I2C4_read(NACK);
    t >>= 4;
    
    if (t & 0x800) {
    t |= 0xF000;
    }

    float temp = t;
    temp /= 16.0;
    return temp;
}

/*!
 *  @brief  read 1 byte of data at the specified address
 *  @param  a 
 *          the address to read
 *  @return the read data byte
 */
uint8_t read8(uint8_t a) {
    uint8_t inbyte;
    I2C4_start();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x00); // start transmission to device
    I2C4_write(a);               // sends register address to read from
    I2C4_restart();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x01);
    inbyte = I2C4_read(NACK); // receive DATA  
    I2C4_stop();
    return inbyte;
}

/*!
 *  @brief  write a byte of data to the specified address
 *  @param  a 
 *          the address to write to
 *  @param  d 
 *          the byte to write
 */
void write8(uint8_t a, uint8_t d) {
    I2C4_start();
    I2C4_write((MPL3115A2_ADDRESS<<1)|0x00); // start transmission to device
    I2C4_write(a);               // sends register address to read from
    I2C4_write(d);               // sends register data
    I2C4_stop();              // end transmission
}